/*
 * A pager abstraction for handling map requests.
 *
 * Copyright (C) 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/accessor.h>
#include <fsserver/flexpage.h>
#include <fsserver/pages.h>



/* A pager exporting a region of a mapped resource. Resources inherit from
   pagers and provide accessors in order to provide paging. */

class Pager
{
public:
  /* Flexpage handling. */

  void flush_all(bool keep=false);

  /* Access to data. */

  virtual PagingAccessor *get_accessor() = 0;

  /* Exported position and limit accessors. */

  virtual unsigned long get_data_start() = 0;
  virtual unsigned long get_data_size() = 0;

  /* Operation methods. */

  long map(unsigned long offset, l4_addr_t hot_spot, unsigned long flags,
           l4_snd_fpage_t *region);
};
