/*
 * Pipe resource classes.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/accessor.h>
#include <fsserver/pager.h>
#include <fsserver/resource.h>

/* Common interfaces. */

#include <fsserver/pipe_object_interface.h>



/* A pipe resource class. */

class PipeResource : public Resource, public Pager, public PipeObject
{
protected:
  PipePagingAccessor *_accessor;
  bool _writer;

  /* Size of the current region. */

  unsigned long _size;

public:
  explicit PipeResource(PipePagingAccessor *accessor, bool writer, unsigned long size)
  : Resource(), _accessor(accessor), _writer(writer), _size(size)
  {
  }

  /* Server details. */

  int expected_items();

  ipc_server_handler_type handler();

  void *interface()
  { return static_cast<PipeObject *>(this); }

  /* Size access. */

  virtual size_t get_size()
  { return _accessor->get_size(); }

  /* Pager methods. */

  virtual PipePagingAccessor *get_accessor()
  { return _accessor; }

  virtual unsigned long get_data_start()
  { return 0; };

  virtual unsigned long get_data_size()
  { return _size; };

  /* Deallocation of resources. */

  virtual void close();

  /* Activation. */

  virtual void start()
  { _accessor->attach(); }

  /* File operations. */

  virtual long flush(size_t data_start, size_t *size);

  /* Pipe operations. */

  virtual long current_region(size_t *size);

  virtual long next_region(size_t *size, size_t *data_end);

  /* Dataspace operations. */

  virtual long map(unsigned long offset, l4_addr_t hot_spot,
                   unsigned long flags, l4_snd_fpage_t *region)
  { return Pager::map(offset, hot_spot, flags, region); }
};
