/*
 * Common server classes and loops.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>
#include <ipc/message.h>



/* A server object handling incoming messages. */

class Server
{
protected:
  l4_cap_idx_t _cap = L4_INVALID_CAP;
  int _expected_items;

public:
  explicit Server(int expected_items=0)
  : _expected_items(expected_items)
  {
  }

  virtual ~Server()
  {
  }

  /* Capability associated with this object. */

  l4_cap_idx_t cap() { return _cap; }

  /* Number of expected items (capabilities) in incoming messages. */

  int expected_items() { return _expected_items; }

  /* Server binding/registration. */

  virtual long bind(const char *name);

  /* Main loop. */

  virtual long start();

  /* IPC message handling and dispatch methods. */

  virtual void handle(ipc_message_t *msg) = 0;

  virtual void dispatch(ipc_message_t *msg) { (void) msg; }
};



/* Server binding/registration. */

long server_bind(const char *name, Server *obj);

/* Message processing for servers. */

void server_handle(ipc_message_t *msg, Server *obj);

/* Server main loop. */

long server_loop(int expected_items);
