/*
 * User filesystem resource functionality.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/opener_context_resource.h>
#include <fsserver/paging.h>

/* Interface support. */

#include <fsserver/resource.h>
#include <fsserver/buffer_support.h>
#include <fsserver/opener_support.h>

/* Implemented interfaces. */

#include <fsserver/user_filesystem_object_interface.h>



/* A user filesystem class. */

class UserFilesystemResource : public Resource, public UserFilesystemObject,
                               public BufferSupport, public OpenerSupport
{
protected:
  /* Paging object coordination. This is needed for the initialisation of
     filesystem object resources. */

  Paging *_paging;

public:
  explicit UserFilesystemResource(Paging *paging = 0)
  : Resource(), _paging(paging)
  {
  }

  /* Server details. */

  int expected_items();

  ipc_server_handler_type handler();

  void *interface()
  { return static_cast<UserFilesystemObject *>(this); }

  /* Operation methods. */

  virtual long context(l4_cap_idx_t *context)
  {
    return buffer(context);
  }

  virtual long getuserfs(sys_uid_t, sys_gid_t, sys_mode_t, l4_cap_idx_t *)
  {
    return -L4_EACCESS;
  }

  /* Support the construction of an opener context in BufferSupport::buffer. */

  Resource *buffer_resource(void *buffer, unsigned long buffer_size)
  {
    return new OpenerContextResource(buffer, buffer_size, this);
  }
};
