/*
 * Common buffer object creation functionality.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdlib.h> /* posix_memalign */
#include <string.h> /* memset */

#include "buffer_support.h"
#include "resource_server.h"



/* Support the retrieval of a buffer object, delegating the construction of the
   server to a specialised method. */

long BufferSupport::buffer(l4_cap_idx_t *cap)
{
  void *buffer;
  unsigned long buffer_size = L4_PAGESIZE;

  /* NOTE: Allocation should be delegated to a page collection. */

  if (posix_memalign(&buffer, L4_PAGESIZE, buffer_size))
    return -L4_ENOMEM;

  /* Populate the allocated memory to make it available. */

  memset(buffer, 0, buffer_size);

  /* Start a server in a new thread to provide access to the resource. */

  ResourceServer server(buffer_resource(buffer, buffer_size));
  long err = server.start_thread();

  *cap = server.config()->server;
  return err;
}
