/*
 * A simple pager abstraction for handling map requests.
 *
 * Copyright (C) 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/consts.h>

#include <stdio.h>
#include <stdlib.h>

#include <fsserver/flexpage.h>
#include "simple_pager.h"



/* Release allocated resources. */

void SimplePager::close()
{
  if (_buffer != NULL)
  {
    free(_buffer);
    _buffer = NULL;
    _buffer_size = 0;

    printf("Closed simple pager.\n");
  }
}

/* Handle a map request by associating a flexpage with the requested memory
   region. */

long SimplePager::map(unsigned long offset, l4_addr_t hot_spot,
                      unsigned long flags, l4_snd_fpage_t *region)
{
  printf("SimplePager::map(offset=%lx, hot_spot=%lx, flags=%lx)\n",
    offset, hot_spot, flags);

  /* Check the offset. */

  if (offset >= _buffer_size)
    return -L4_ERANGE;

  /* Set the details of the required flexpage. */

  Flexpage flexpage((l4_addr_t) _buffer, _buffer_size);

  flexpage.reset(offset, hot_spot, flags);

  /* Provide memory in this task for use in the caller. */

  region->fpage = flexpage.get_fpage(flags);
  region->snd_base = hot_spot;

  return L4_EOK;
}

/* Handle an info request. */

long SimplePager::info(unsigned long *size, unsigned long *flags)
{
  /* Return details of a read/write buffer of the recorded size. */

  *size = _buffer_size;
  *flags = L4_FPAGE_RW;

  return L4_EOK;
}
